@@ -1,6 +1,8 @@
 use std::io;
 
+use serde::Deserialize;
 use serde_json::json;
+use thiserror::Error;
 
 mod chk {
     // this lets us make the RpcMethod trait public but non-implementable by users outside this crate
@@ -93,10 +95,26 @@ macro_rules! impl_ {
     };
 }
 
+// workaround for deserializing partially serialized
+// error types missing the `error_message` field in
+// their UnknownBlock variants.
+macro_rules! parse_unknown_block {
+    ($json:expr => $err_ty:ident) => {
+        if $json["name"] == "UNKNOWN_BLOCK" {
+            Some(Ok($err_ty::UnknownBlock {
+                error_message: "".to_string(),
+            }))
+        } else {
+            None
+        }
+    };
+}
+
 mod shared_impls {
     use super::{RpcHandlerError, RpcHandlerResponse};
 
-    // broadcast_tx_async, EXPERIMENTAL_genesis_config, adv_*
+    // adv_*
+    #[cfg(feature = "adversarial")]
     impl RpcHandlerError for () {}
 
     // adv_*
@@ -141,7 +159,11 @@ mod shared_impls {
     impl RpcHandlerError for near_jsonrpc_primitives::types::status::RpcStatusError {}
 
     // EXPERIMENTAL_changes, EXPERIMENTAL_changes_in_block
-    impl RpcHandlerError for near_jsonrpc_primitives::types::changes::RpcStateChangesError {}
+    impl RpcHandlerError for near_jsonrpc_primitives::types::changes::RpcStateChangesError {
+        fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+            parse_unknown_block!(value => Self)
+        }
+    }
 
     // EXPERIMENTAL_broadcast_tx_sync, EXPERIMENTAL_check_tx
     impl RpcHandlerResponse
@@ -228,7 +250,11 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcBlockResponse {}
 
-        impl RpcHandlerError for RpcBlockError {}
+        impl RpcHandlerError for RpcBlockError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcBlockRequest {
             type Response = RpcBlockResponse;
@@ -270,11 +296,17 @@ impl_method! {
             }
         }
 
+        #[derive(Debug, Deserialize, Error)]
+        #[error("{}", unreachable!("fatal: this error should never constructed"))]
+        pub enum RpcBroadcastTxAsyncError {}
+
         impl RpcHandlerResponse for RpcBroadcastTxAsyncResponse {}
 
+        impl RpcHandlerError for RpcBroadcastTxAsyncError {}
+
         impl_!(RpcMethod for RpcBroadcastTxAsyncRequest {
             type Response = RpcBroadcastTxAsyncResponse;
-            type Error = ();
+            type Error = RpcBroadcastTxAsyncError;
 
             fn params(&self) -> Result<serde_json::Value, io::Error> {
                 Ok(json!([serialize_signed_transaction(&self.signed_transaction)?]))
@@ -324,14 +356,18 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcChunkResponse {}
 
-        impl RpcHandlerError for RpcChunkError {}
+        impl RpcHandlerError for RpcChunkError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcChunkRequest {
             type Response = RpcChunkResponse;
             type Error = RpcChunkError;
 
             fn params(&self) -> Result<serde_json::Value, io::Error> {
-                Ok(json!([self]))
+                Ok(json!(self))
             }
         });
     }
@@ -347,7 +383,11 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcGasPriceResponse {}
 
-        impl RpcHandlerError for RpcGasPriceError {}
+        impl RpcHandlerError for RpcGasPriceError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcGasPriceRequest {
             type Response = RpcGasPriceResponse;
@@ -391,7 +431,11 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcLightClientExecutionProofResponse {}
 
-        impl RpcHandlerError for RpcLightClientProofError {}
+        impl RpcHandlerError for RpcLightClientProofError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcLightClientExecutionProofRequest {
             type Response = RpcLightClientExecutionProofResponse;
@@ -414,7 +458,11 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcLightClientNextBlockResponse {}
 
-        impl RpcHandlerError for RpcLightClientNextBlockError {}
+        impl RpcHandlerError for RpcLightClientNextBlockError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcLightClientNextBlockRequest {
             type Response = RpcLightClientNextBlockResponse;
@@ -429,9 +477,7 @@ impl_method! {
 
 impl_method! {
     pub mod network_info {
-        pub use near_jsonrpc_primitives::types::network_info::RpcNetworkInfoError;
-
-        pub type RpcNetworkInfoResponse = near_client_primitives::types::NetworkInfoResponse;
+        pub use near_jsonrpc_primitives::types::network_info::{RpcNetworkInfoError, RpcNetworkInfoResponse};
 
         #[derive(Debug)]
         pub struct RpcNetworkInfoRequest;
@@ -666,11 +712,17 @@ impl_method! {
         #[derive(Debug)]
         pub struct RpcGenesisConfigRequest;
 
+        #[derive(Debug, Deserialize, Error)]
+        #[error("{}", unreachable!("fatal: this error should never constructed"))]
+        pub enum RpcGenesisConfigError {}
+
         impl RpcHandlerResponse for RpcGenesisConfigResponse {}
 
+        impl RpcHandlerError for RpcGenesisConfigError {}
+
         impl_!(RpcMethod for RpcGenesisConfigRequest {
             type Response = RpcGenesisConfigResponse;
-            type Error = ();
+            type Error = RpcGenesisConfigError;
 
             fn params(&self) -> Result<serde_json::Value, io::Error> {
                 Ok(json!(null))
@@ -689,7 +741,11 @@ impl_method! {
 
         impl RpcHandlerResponse for RpcProtocolConfigResponse {}
 
-        impl RpcHandlerError for RpcProtocolConfigError {}
+        impl RpcHandlerError for RpcProtocolConfigError {
+            fn parse_raw_error(value: serde_json::Value) -> Option<Result<Self, serde_json::Error>> {
+                parse_unknown_block!(value => Self)
+            }
+        }
 
         impl_!(RpcMethod for RpcProtocolConfigRequest {
             type Response = RpcProtocolConfigResponse;
